<?php
/*
Plugin Name: Public Chat
Plugin URI: https://www.drivebird.com/public-chat
Description: A simple and standalone public chat page powered by global public chat network.
Author: Atanu Biswas
Version: 20230323
Author URI: https://atanubiswas.in
License: GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
*/

defined('ABSPATH') OR exit('No direct script access allowed');

// PUBLIC CHAT PAGE
add_filter( 'init', function( $template ) {
    $url = DBNPublicChat_current_url();

    $settings = DBNPublicChat_db_data();

    if ($url == $settings['dsu']) {
        $params = "";
        if(!empty($settings))
        {
            $params = '&dsu='.$url;
            if(!empty($settings['brd']))
                $params .= '&brd='.urlencode($settings['brd']);
            if(!empty($settings['hbu']))
                $params .= '&hbu='.urlencode($settings['hbu']);
            if(!empty($settings['gac']))
                $params .= '&gac='.$settings['gac'];
            if(!empty($settings['hdn']))
                $params .= '&hdn='.$settings['hdn'];
            if(!empty($settings['hdd']))
                $params .= '&hdd='.$settings['hdd'];
        }

        echo wp_remote_retrieve_body( wp_remote_get('https://public-chat.drivebird.com/?'.$params) );
        die;
    }
} );

//----------PLUGIN-------------//

// MENU
add_action( 'admin_menu', 'pub_chat_admin_menu' );
function pub_chat_admin_menu() {
    // add top level menu page
    add_menu_page(
        'Public Chat',
        'Public Chat',
        'manage_options',
        'hello-all',
        'DBNPublicChat_save_settings'
    );
}

// ALL SETTINGS PAGE COMPONENTS
add_action( 'admin_init', 'pub_chat_settings_init' );
function pub_chat_settings_init() {
    register_setting( 'DBN_PublicChat', 'DBNPublicChat_options' );

    add_settings_section(
        'dbn_section_developers',
        __( 'Customize your public chat page', 'DBN_PublicChat' ),
        'DBNPublicChat_developer_contact',
        'DBN_PublicChat'
    );

    // FORM FIELD - HIDE DONATION
    add_settings_field(
        'DBNPublicChat_hide_donation',
        'Hide donation',
        'DBNPublicChat_hide_donation_field',
        'DBN_PublicChat',
        'dbn_section_developers',
        [
            'label_for' => 'DBNPublicChat_hide_donation',
            'class' => 'DBNPublicChat_row',
        ]
    );

    // FORM FIELD - HIDE DONATION
    add_settings_field(
        'DBNPublicChat_hide_developer_documentation',
        'Hide developer documentation',
        'DBNPublicChat_hide_developer_documentation_field',
        'DBN_PublicChat',
        'dbn_section_developers',
        [
            'label_for' => 'DBNPublicChat_hide_developer_documentation',
            'class' => 'DBNPublicChat_row',
        ]
    );

    // FORM FIELD - BRAND NAME
    add_settings_field(
        'DBNPublicChat_brand_name',
        'Brand name',
        'DBNPublicChat_brand_name_field',
        'DBN_PublicChat',
        'dbn_section_developers',
        [
            'label_for' => 'DBNPublicChat_brand_name',
            'class' => 'DBNPublicChat_row',
        ]
    );

    // FORM FIELD - GOOGLE ANALYTICS CODE
    add_settings_field(
        'DBNPublicChat_ga_code',
        'Google analytics code',
        'DBNPublicChat_google_analytics_field',
        'DBN_PublicChat',
        'dbn_section_developers',
        [
            'label_for' => 'DBNPublicChat_ga_code',
            'class' => 'DBNPublicChat_row',
        ]
    );

    // FORM FIELD - HOME OR BACK URL
    add_settings_field(
        'DBNPublicChat_home_back_url',
        'Home or back URL',
        'DBNPublicChat_home_back_url_field',
        'DBN_PublicChat',
        'dbn_section_developers',
        [
            'label_for' => 'DBNPublicChat_home_back_url',
            'class' => 'DBNPublicChat_row',
        ]
    );

    // FORM FIELD - DISPLAY SITE URL
    add_settings_field(
        'DBNPublicChat_display_site_url',
        'Chat display URL',
        'DBNPublicChat_display_site_url_field',
        'DBN_PublicChat',
        'dbn_section_developers',
        [
            'label_for' => 'DBNPublicChat_display_url_key',
            'class' => 'DBNPublicChat_row',
        ]
    );
}


//---------COMPONENT FUNCTIONS --------------//

// DEVELOPER CONTACT TEXT
function DBNPublicChat_developer_contact( $args ) {
    ?>
    <p id="<?php echo esc_attr( $args['id'] ); ?>">For any assistance you can visit <a target="_blank" href="https://www.drivebird.com/public-chat">developer documentation</a> page or email me at <a href="mailto:drivebird@drivebird.com?subject=WP support needed">drivebird@drivebird.com</a>.</p>
    <?php
}


// FORM FIELD - HIDE DONATION
function DBNPublicChat_hide_donation_field( $args )
{
    $dbData = DBNPublicChat_db_data();
    ?>
    <select id="DBNPublicChat_hdn" name="DBNPublicChat_options[DBNPublicChat_hdn]">
        <option value="1" <?php echo ($dbData['hdn'] == '1')? 'selected':'';?>><?php esc_html_e( 'Yes', 'DBN_PublicChat' ); ?></option>
        <option value="0" <?php echo ($dbData['hdn'] == '0')? 'selected':'';?>><?php esc_html_e( 'No', 'DBN_PublicChat' ); ?></option>
    </select>
    <p class="description">
        <?php esc_html_e( 'Select Yes to hide donation widget', 'DBN_PublicChat' ); ?>
    </p>
    <?php
}

// FORM FIELD - HIDE DEVELOPER DOCUMENTATION
function DBNPublicChat_hide_developer_documentation_field( $args )
{
    $dbData = DBNPublicChat_db_data();
    ?>
    <select id="DBNPublicChat_hdd" name="DBNPublicChat_options[DBNPublicChat_hdd]">
        <option value="1" <?php echo ($dbData['hdd'] == '1')? 'selected':'';?>><?php esc_html_e( 'Yes', 'DBN_PublicChat' ); ?></option>
        <option value="0" <?php echo ($dbData['hdd'] == '0')? 'selected':'';?>><?php esc_html_e( 'No', 'DBN_PublicChat' ); ?></option>
    </select>
    <p class="description">
        <?php esc_html_e( 'Select Yes to hide developer documentation widget', 'DBN_PublicChat' ); ?>
    </p>
    <?php
}

// FORM FIELD - BRAND NAME
function DBNPublicChat_brand_name_field( $args )
{
    $dbData = DBNPublicChat_db_data();
    ?>
    <input type="text" name="DBNPublicChat_options[DBNPublicChat_brd]" value="<?php echo $dbData['brd'];?>" />
    <p class="description">
        <?php esc_html_e( 'Brand or title of your chat page', 'DBN_PublicChat' ); ?>
    </p>
    <?php
}

// FORM FIELD - GOOGLE ANALYTICS CODE
function DBNPublicChat_google_analytics_field( $args )
{
    $dbData = DBNPublicChat_db_data();
    ?>
    <input type="text" name="DBNPublicChat_options[DBNPublicChat_gac]" value="<?php echo $dbData['gac'];?>" />
    <p class="description">
        <?php esc_html_e( 'Google analytics code (Ex: UA-XXXXXXXX-X). Put \'#\' to disable', 'DBN_PublicChat' ); ?>
    </p>
    <?php
}

// FORM FIELD - HOME OR BACK URL
function DBNPublicChat_home_back_url_field( $args )
{
    $dbData = DBNPublicChat_db_data();
    ?>
    <input type="text" name="DBNPublicChat_options[DBNPublicChat_hbu]" value="<?php echo $dbData['hbu'];?>" />
    <p class="description">
        <?php esc_html_e( 'Home or back URL. Put \'#\' if you dont want to use \'Back\' button', 'DBN_PublicChat' ); ?>
    </p>
    <?php
}

// FORM FIELD - DISPLAY SITE URL
function DBNPublicChat_display_site_url_field( $args )
{
    $dbData = DBNPublicChat_db_data();
    ?>
    <input type="text" name="DBNPublicChat_options[DBNPublicChat_dsu]" value="<?php echo $dbData['dsu'];?>" />
    <p class="description">
        <?php esc_html_e( 'Full URL of chat display page', 'DBN_PublicChat' ); ?>
    </p>
    <?php
}

// THIS RETURNS CURRENT URL
function DBNPublicChat_current_url()
{
    $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || @$_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
    $url = (isset($_SERVER['HTTP_HOST']) && (isset($_SERVER['REQUEST_URI'])))? $protocol.$_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']:"";
    return $url;
}

// THIS RETURNS OBJECT FROM JSON STORAGE
function DBNPublicChat_db_data()
{
    // DEFAULT CONFIGURATION | FALLBACK
    $db['brd'] = get_bloginfo( 'name' ); // BRAND
    $db['gac'] = "#"; // GOOGLE ANALYTICS CODE
    $db['hdn'] = "0"; // HIDE DONATION
    $db['hdd'] = "0"; // HIDE DEVELOPER DOCUMENTATION
    $db['hbu'] = site_url(); // HOME URL / BACK URL
    $db['dsu'] = site_url("public-chat"); // DISPLAY SITE URL

    $settingsFile = DBNPublicChat_settings_file_path();
    $settings = @json_decode(@base64_decode(@file_get_contents($settingsFile)),true,10);

    if((!empty($settings)) && (is_array($settings)))
    {
        $settings = array_filter($settings);
        $db = array_merge($db,$settings);
    }

    return $db;
}

// RETURNS CHAT URL
function DBNPublicChat_chat_url()
{
    $dbData = DBNPublicChat_db_data();
    return $dbData['dsu'];
}

// CHECK IF A STRING IS JSON OR NOT
function DBNPublicChat_is_json($str)
{
    if(empty($str))
        return false;
    else
        return true;
}

// RETURNS DATA FROM ARRAY ELEMENT
function DBNPublicChat_get_data($array, $key)
{
    return isset($array[$key]) ? $array[$key] : "";
}

// RETURNS SETTINGS FILE PATH
function DBNPublicChat_settings_file_path()
{
    return wp_upload_dir('chat')['path'].md5(__FILE__).'_DBNPublicChat_settings.json';
}

// FORM TEMPLATE MARKUP
function DBNPublicChat_form_template_markup()
{
    ?>
    <div class="wrap">
        <h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <form action="<?php echo DBNPublicChat_current_url();?>" method="post">
            <input type="hidden" name="action" value="DBN_PublicChat">
            <?php
            do_settings_sections( 'DBN_PublicChat' );
            submit_button( 'Save Settings' );
            ?>
        </form>
    </div>
    <?php
}

// SAVE SETTINGS
function DBNPublicChat_save_settings()
{
    // check user capabilities
    if ( ! current_user_can( 'manage_options' ) ) {
        return;
    }

    $url = DBNPublicChat_current_url();

    // SUBMIT FOUND
    if(isset($_POST["action"]))
    {
        if($_POST["action"] == "DBN_PublicChat")
        {
            $dbData = DBNPublicChat_db_data();
            $dbData['hdn'] = DBNPublicChat_get_data($_POST['DBNPublicChat_options'], 'DBNPublicChat_hdn');
            $dbData['hdd'] = DBNPublicChat_get_data($_POST['DBNPublicChat_options'], 'DBNPublicChat_hdd');
            $dbData['brd'] = DBNPublicChat_get_data($_POST['DBNPublicChat_options'], 'DBNPublicChat_brd');
            $dbData['gac'] = DBNPublicChat_get_data($_POST['DBNPublicChat_options'], 'DBNPublicChat_gac');
            $dbData['hbu'] = DBNPublicChat_get_data($_POST['DBNPublicChat_options'], 'DBNPublicChat_hbu');
            if(strtolower(DBNPublicChat_get_data($_POST['DBNPublicChat_options'], 'DBNPublicChat_dsu')) == $url)
                add_settings_error( 'dbn_messages', 'dbn_message', __( 'Chat page can not be set as admin URL', 'DBN_PublicChat' ), 'error' );
            else
                $dbData['dsu'] = str_replace('_','-',strtolower(DBNPublicChat_get_data($_POST['DBNPublicChat_options'], 'DBNPublicChat_dsu')));
            $settingsFile = DBNPublicChat_settings_file_path();
            if(@file_put_contents($settingsFile, base64_encode(json_encode($dbData))))
                add_settings_error( 'dbn_messages', 'dbn_message', __( 'Settings saved', 'DBN_PublicChat' ), 'updated' );
            else
                add_settings_error( 'dbn_messages', 'dbn_message', __( 'Settings not saved', 'DBN_PublicChat' ), 'error' );
        }
    }

    // SHOW CHAT URL
    $chatUrl = DBNPublicChat_chat_url();
    if(!empty($chatUrl))
        add_settings_error( 'dbn_messages', 'dbn_message', __( 'Public Chat URL: <a target="_blank" href="'.$chatUrl.'">'.$chatUrl.'</a>', 'DBN_PublicChat' ), 'success' );

    // SHOWING MESSAGE IF IN THE MEMORY
    settings_errors( 'dbn_messages' );
    DBNPublicChat_form_template_markup();
}
?>
